//
// Copyright 2015-2021 by Garmin Ltd. or its subsidiaries.
// Subject to Garmin SDK License Agreement and Wearables
// Application Developer Agreement.
//

import Toybox.Graphics;
import Toybox.Lang;
import Toybox.WatchUi;

public enum Align {
    ALIGN_BOTTOM_RIGHT,
    ALIGN_BOTTOM_LEFT,
    ALIGN_BOTTOM_CENTER,
    ALIGN_TOP_RIGHT,
    ALIGN_TOP_LEFT,
    ALIGN_TOP_CENTER
}

//! Draws a graphic indicating which page the user is currently on
class PageIndicator {
    private var _size as Number;
    private var _selectedColor as ColorType;
    private var _notSelectedColor as ColorType;
    private var _alignment as Align;
    private var _margin as Number;

    //! Constructor
    //! @param size Number of pages
    //! @param selectedColor Color to use for the selected page
    //! @param notSelectedColor Color to use for non-selected pages
    //! @param alignment How to align the graphic
    //! @param margin Amount of margin for the graphic
    public function initialize(size as Number, selectedColor as ColorValue, notSelectedColor as ColorValue, alignment as Align, margin as Number) {
        C.logger("page:initialize()");
        _size = size;
        _selectedColor = selectedColor;
        _notSelectedColor = notSelectedColor;
        _alignment = alignment;
        _margin = margin;
    }

    //! Draw the graphic
    //! @param dc Device context
    //! @param selectedIndex The index of the current page
    public function draw(dc as Dc, selectedIndex as Number) as Void {
        var height = 10;
        var width = _size * height;
        var x = 0;
        var y = 0;

        C.logger("page:draw - " + selectedIndex);

        if (_alignment == $.ALIGN_BOTTOM_RIGHT) {
            x = dc.getWidth() - width - _margin;
            y = dc.getHeight() - height - _margin;
        } else if (_alignment == $.ALIGN_BOTTOM_LEFT) {
            x = 0 + _margin;
            y = dc.getHeight() - height - _margin;
        } else if (_alignment == $.ALIGN_BOTTOM_CENTER) {
            x = (dc.getWidth() / 2) - (width / 2);
            y = dc.getHeight() - height - _margin;
        }  else if (_alignment == $.ALIGN_TOP_RIGHT) {
            x = dc.getWidth() - width - _margin;
            y = 0 + _margin;
        } else if (_alignment == $.ALIGN_TOP_LEFT) {
            x = 0 + _margin;
            y = 0 + _margin;
        } else if (_alignment == $.ALIGN_TOP_CENTER) {
            x = (dc.getWidth() / 2) - (width / 2);
            y = 0 + _margin;
        }

        for (var i = 0; i < _size; i++) {
            if (i == selectedIndex) {
                dc.setColor(_selectedColor, Graphics.COLOR_TRANSPARENT);
            } else {
                dc.setColor(_notSelectedColor, Graphics.COLOR_TRANSPARENT);
            }

            var tempX = (x + (i * height)) + height / 2;
            dc.fillCircle(tempX, y, height / 2);
        }
    }

}

//! ViewLoop Delegate for handling the main views
//! and indicates the current page
class PageIndicatorDelegate extends WatchUi.ViewLoopDelegate {

    private var mViewLoop;

    //! Constructor
    //! @param viewLoop The ViewLoop Object
    function initialize(viewLoop) {
        C.logger("loop:initialize()");
        ViewLoopDelegate.initialize(viewLoop);
        mViewLoop = viewLoop;
    }

    //! Handle going to the next view
    //! @return true if handled, false otherwise
    function onNextView() {
        C.logger("loop:onNextView()");
        mViewLoop.changeView(WatchUi.ViewLoop.DIRECTION_NEXT);
        return true;
    }

    //! Handle going to the previous view
    //! @return true if handled, false otherwise
    function onPreviousView() {
        C.logger("loop:onPreviousView()");
        mViewLoop.changeView(WatchUi.ViewLoop.DIRECTION_PREVIOUS);
        return true;
    }
}

//! ViewLoop Factory which manages the main view/delegate pairs
class PageIndicatorFactory extends WatchUi.ViewLoopFactory {

function initialize()
{

    ViewLoopFactory.initialize();
}

//! Retrieve a view/delegate pair for the page at the given index
function getView(page as Number) as Array<View or BehaviorDelegate>?
{

    return [new $.AmbitView(page), new $.AmbitViewLoopDelegate(page)] as Array<Views or InputDelegates>;
}

//! Return the number of view/delegate pairs that are managed by this factory
function getSize()
{

    return C.NUM_VIEWS;
}

}
